#version 430 core
layout (location=0) out vec4 FragColor;
  
in vec2 TexCoords;

uniform sampler2D depthTexture;

//Kernels for edge detection
float sy[9] = float[](
	-1.0, -2.0, -1.0,
	0.0, 0.0, 0.0,
	1.0, 2.0, 1.0
	);


float sx[9] = float[](
	1.0, 0.0, -1.0,
	2.0, 0.0, -2.0,
	1.0, 0.0, -1.0
);


float near = 0.1;
float far = 100.0;

//linearize to get right values
float linearize(float depth){
	float z = depth * 2.0 - 1.0;
	return ((2.0 * near * far) / (far + near - z * (far - near)));
}


void main()
{ 

	vec2 offset = 1.0 / textureSize(depthTexture, 0);

	//offset array for neighbour pixel
	vec2 offsets[9] = vec2[] (
        vec2(-offset.x, offset.y),	// top-left
        vec2(0.0f, offset.y),		// top-center
        vec2(offset.x, offset.y),	// top-right
        vec2(-offset.x, 0.0f),		// center-left
        vec2( 0.0f,  0.0f),			// center-center
        vec2( offset.x, 0.0f),		// center-right
        vec2(-offset.x, -offset.y), // bottom-left
        vec2( 0.0f, -offset.y),		// bottom-center
        vec2( offset.x, -offset.y)  // bottom-right    
	);

	float I[9];
	for(int i = 0; i < 9; i++){
		I[i] = linearize(texture(depthTexture, TexCoords.xy + offsets[i]).r)/far;

	}


	float gx = 0.0;
	float gy = 0.0;

	for(int i = 0; i < 9; i++){
		 gx += I[i] * sx[i];
		 gy += I[i] * sy[i];
	}

	float g = sqrt(pow(gx, 2.0) + pow(gy, 2.0));
	
	g = smoothstep(0.01, 0.1, g);

	FragColor = vec4(vec3(g), 1.0);




	
}


