#version 430
// It was expressed that some drivers required this next line to function properly
precision highp float;

layout(location = 1) uniform mat4 mvpMatrix;
layout(location = 2) uniform mat4 normalMatrix;
layout(location = 3) uniform mat4 mvMatrix;
layout(location = 4) uniform mat4 viewMatrix;

layout(location=0) uniform  vec3 color;

layout(location=5) uniform float diffuse;
layout(location=6) uniform float ambient;
layout(location=7) uniform float specular;
layout(location=8) uniform float alpha;
layout(location=9) uniform float brightness;
layout(location=10) uniform bool hasTexture;
layout(location=11) uniform bool hasLightmap;
layout(location=12) uniform bool hasNormalmap;
layout(location=13) uniform bool hasEmissionmap;
layout(location=14) uniform float uvscale=1.0;
layout(location=15) uniform bool normalMappingEnabled;

in vec3 n;
in vec3 fragmentPosition;
in vec2 TexCoord;
in vec2 LMCoord;
in mat3 TBN;

uniform sampler2D colorTexture;
uniform sampler2D lightmapTexture;
uniform sampler2D normalTexture;
uniform sampler2D emissionTexture;

const float levels = 3.0;

layout(location=0) out vec4 fragColor;
layout(location=1) out vec4 emissionColor;






uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;


struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
};

uniform PointLight pointlights[10];
uniform int pointlightcount = 0;

vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation) {
	float d = length(l);
	l = normalize(l);
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);
	float gradient = max(0, dot(n, l));

	return (diffuseF * diffuseC * specularC * ceil(gradient*levels)/levels + specularF * specularC * smoothstep(0.48, 0.52, pow(max(0, dot(r, v)), alpha))) * att; 
}

void main(void) {
	vec3 v = normalize(-fragmentPosition);
	vec3  texcolor = color;
	if(hasTexture){
		texcolor = texture(colorTexture, TexCoord * uvscale).rgb;
	} 

	vec3 normal;
	if(hasNormalmap && normalMappingEnabled){
		normal = texture(normalTexture,TexCoord*uvscale).rgb;
		normal = normal*2-1;
		normal = TBN*normal;
	} else {
		normal = normalize(n);
	}

	vec3 lightdir = -normalize((viewMatrix * vec4(dirL.direction, 0)).xyz);
	
	vec3 combined = vec3(0);
	for(int i = 0; i< 10 && i < pointlightcount; i++)
	{
		vec3 pointpos = (viewMatrix * vec4(pointlights[i].position, 1)).xyz;
		vec3 pointdir = pointpos- fragmentPosition;
		
		combined += phong(normal, pointdir, v, texcolor, diffuse, pointlights[i].color, specular, alpha, true, pointlights[i].attenuation);
	}
	

	if(!hasLightmap){
		combined += phong(normal, lightdir, v, texcolor, diffuse, dirL.color, specular, alpha, false, vec3(0)) + (ambient+brightness)*texcolor;

		
	}
	else {
		combined += texture(lightmapTexture, LMCoord).rgb * texcolor + (ambient+brightness)*texcolor;
		}
	
	fragColor = vec4(combined, 1.0);

	
	
	// Emission
	if(hasEmissionmap) {
		vec4 em = texture(emissionTexture, TexCoord);
		fragColor = mix(fragColor, vec4(color,1),em.x);
		emissionColor = vec4(color,1.0)*em;
	} else {
		emissionColor = vec4(0.0,0.0,0.0,1.0);
	}

	

	
}